==> lowcode-render-0.2.15-alpha.0.tgz: package/src/utils/common.ts <==
import { IPlatformApp, IPlatformComponent } from '@cloudbase/cals';
import { widgetKeys } from './widgets';
import { app as _app, IAppAPI } from '../app/global-api';
import { auth } from '@cloudbase/weda-client';
const { loginScope, getAccessToken } = auth;

/**
 * All data bindings are generated as functions: (forItems) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(dataBinds, forItems, codeContext, scopeContext, throwError, dataContext) {
  const resolvedProps = {};
  for (const prop in dataBinds) {
    let fn = dataBinds[prop];
    try {
      if (codeContext?.$WEAPPS_COMP) {
        fn = fn.bind(codeContext.$WEAPPS_COMP);
      }
      resolvedProps[prop] = fn(forItems, codeContext?.event, dataContext);
    } catch (e) {
      console.error('Error resolving data binding', prop, dataBinds[prop], e);
      if (throwError) {
        throw e;
      }
    }
  }
  return resolvedProps;
}

/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj = {}, context) {
  return Object.keys(funcObj).reduce((obj, fnName) => {
    obj[fnName] = funcObj[fnName]?.bind?.(context);
    return obj;
  }, {});
}

export function kebabCase(str) {
  return str.replace(/[A-Z]/g, function (match) {
    return `-${match.toLowerCase()}`;
  });
}

export function camelcase(str, firstUpperCase = false) {
  str = str.replace(/[_-]([a-z])/g, function (l) {
    return l[1].toUpperCase();
  });

  if (firstUpperCase) str = str.charAt(0).toUpperCase() + str.slice(1);

  return str;
}

export const isEmptyObj = (obj) => {
  if (!isPlainObject(obj)) {
    return false;
  }
  for (const i in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, i)) {
      return false;
    }
  }
  return true;
};

export function isPlainObject(src) {
  return Object.prototype.toString.call(src) === '[object Object]';
}

const SCOPE_SLOT_COMPONENT_LIB = ['tea_basis', 'tea_shop', 'crm_basis'];

const SCOPE_SLOT_MAP = SCOPE_SLOT_COMPONENT_LIB.reduce((map, lib) => {
  map[`${lib}:TableMatching`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TableExpanded`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TabsTable`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:Table`] = {
    headerSlot: true,
    recordSlot: true,
  };

  return map;
}, {});

export function isSlot(comp: IPlatformComponent) {
  return !comp['module'] && !comp.component;
}

/**
 * 判断是否为作用域插槽
 * 1、根据组件dataForm是否有属性包含 x-slot-scope 属性
 * 2、保持历史穷举法不变
 */
export function isScopeSlot(
  comp: {
    module: string;
    component: string;
  },
  slot: string,
  context?: { componentsInfoMap: any } | any,
) {
  const sourceKey = comp.module && comp.component && `${comp.module}:${comp.component}`;

  let hasXslotScope = false;
  try {
    const dataForm = context?.componentsInfoMap?.[sourceKey]?.dataForm;
    if (typeof dataForm === 'object') {
      hasXslotScope = Object.keys(dataForm).some(
        (prop) => slot.startsWith(`${prop}_`) && dataForm[prop]?.['x-slot-scope'],
      );
    }
  } catch (e) {}

  const map = SCOPE_SLOT_MAP[sourceKey];
  return hasXslotScope || map?.[slot];
}

export function findLoginPage(app = _app) {
  const { pages = [] } = app.__internal__.getConfig();
  return pages.find((item) => item.type === 'login');
}

let _AUTH_CONFIG_CACHE = null;
export async function getAuthConfig(app = _app) {
  if (_AUTH_CONFIG_CACHE) {
    return _AUTH_CONFIG_CACHE;
  }
  try {
    const res = await app.cloud.callWedaApi({
      action: 'DescribeRuntimeResourceStrategy',
      data: {
        ResourceType: 'app',
        ResourceId: app.id,
      },
    });
    const settingData = {};
    // 云api不支持map只能传字符串，需要转换
    res.forEach((item) => {
      settingData[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
    });
    _AUTH_CONFIG_CACHE = settingData;
    return _AUTH_CONFIG_CACHE;
  } catch (e) {
    return {
      NeedLogin: false,
      RejectStrategy: 'show_warning',
    };
  }
}

let _AUTH_CACHE_MAP = {};
async function getAccessPermission(app, appId, pageId) {
  const cacheKey = `${appId}-${pageId}`;
  if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
    return _AUTH_CACHE_MAP[cacheKey];
  }

  const res = await app.cloud.callWedaApi({
    action: 'DescribeResourcesPermission',
    data: {
      ResourceType: `app`,
      ResourceIdList: [cacheKey],
      AppResourceId: appId,
    },
  });
  if (Array.isArray(res) && res.length > 0) {
    const isAccess = !!res[0].IsAccess;
    _AUTH_CACHE_MAP[cacheKey] = isAccess;
    return isAccess;
  } else {
    throw new Error(`鉴权接口返回参数不正确 - ${JSON.stringify(res)}`);
  }
}

export async function checkAnonymous() {
  // 用户是否非匿名登录前端判断条件：有accessToken且scope不为anonymous
  // 此分支逻辑本不应该前端判断是否登录，历史原因后端短期内搞不定，后续后端优化后删除
  let isAnonymous = true;
  try {
    const [scope, { accessToken }] = await Promise.all([loginScope(), getAccessToken()]);
    if (accessToken && scope !== 'anonymous') {
      isAnonymous = false;
    }
  } catch (e) {}
  return isAnonymous;
}

export function formatErrorMsg(e) {
  let msg = e?.error_description || e?.message;
  const uri = e?.error_uri;
  msg = `${msg}${uri ? '-' + uri : ''}`;
  if (msg.startsWith('Failed to fetch')) {
    msg = `网络故障, 请检查本地网络连接是否正常(${msg})`;
  }
  return msg;
}

/**
 * 检查页面权限
 **/
export async function checkAuth(app, appId, $page, generateLoginUrl?: GenerateLoginUrlType) {
  app.showNavigationBarLoading();
  const { loginConfigVersion } = app.__internal__.getConfig();
  const requestList = [getAccessPermission(app, appId, $page.id)];
  // 有登录配置版本号时，才拉取自定义登录配置进行校验
  if (loginConfigVersion) {
    requestList.push(getAuthConfig(app));
    requestList.push(checkAnonymous());
  }
  try {
    const [isAccess, authConfig, isAnonymous] = await Promise.all(requestList);
    app.hideNavigationBarLoading();

    if (loginConfigVersion && isAnonymous) {
      if (authConfig.NeedLogin || (!isAccess && authConfig.RejectStrategy === 'to_login')) {
        redirectToLogin(generateLoginUrl);
        return false;
      }
    }
  
    if (!isAccess) {
      app.showToast({
        title: '页面无访问权限',
        icon: 'error',
      });
    }
    return isAccess;
  } catch(e) {
    app.showModal({
      title: '页面鉴权失败',
      content: formatErrorMsg(e),
      showCancel: false,
    });
    // @ts-ignore
    window._aegis?.error('CUSTOM_ERROR:', e);
    return false;
  }
}

export type GenerateLoginUrlType = (ctx: { app: IAppAPI }, originFn: () => string) => string;

export const defaultGenerateLoginUrl: GenerateLoginUrlType = () => {
  return getLoginUrl();
};

function getLoginUrl() {
  const app = _app;
  const { envId, id, clientId, loginConfigVersion } = app.__internal__.getConfig();
  return `${location.origin}/__auth/?redirect_uri=${encodeURIComponent(
    location.href,
  )}&app_id=${id}&env_id=${envId}&client_id=${clientId}&config_version=${loginConfigVersion}`;
}

export async function redirectToLogin(generateLoginUrl: GenerateLoginUrlType) {
  // 去登录则清空权限缓存。
  _AUTH_CACHE_MAP = {};
  const app = _app;

  const { loginConfigVersion } = app.__internal__.getConfig();
  // 有登录配置版本号时，自定义登录开启，则跳转到自定义登录页面
  if (loginConfigVersion) {
    const url = generateLoginUrl({ app }, getLoginUrl);
    app.redirectTo({
      mode: 'web',
      params: {},
      url,
    });
  } else {
    app.showToast({
      title: '用户未登录',
      icon: 'error',
    });
  }
}

export function generateDataContext(widget) {
  const dataContext = {};
  while (widget?._scope) {
    const current = widget;
    // 此处采用代理的方式，是为了可以获取到最新的 _scope.dataContext 防止 dataContext 引用被重新赋值
    Object.defineProperty(dataContext, current._scope.id, {
      get() {
        return current._scope.dataContext;
      },
    });
    widget = widget.parent;
  }
  return dataContext;
}

==> lowcode-render-0.2.20-alpha.0.tgz: package/src/utils/common.ts <==
import { IPlatformComponent } from '@cloudbase/cals';
import { app as _app, IAppAPI } from '../app/global-api';
import { auth } from '@cloudbase/weda-client';
const { loginScope, getAccessToken } = auth;

/**
 * All data bindings are generated as functions: (forItems) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(dataBinds, forItems, codeContext, scopeContext, throwError, dataContext, $w) {
  const resolvedProps = {};
  for (const prop in dataBinds) {
    let fn = dataBinds[prop];
    try {
      if (codeContext?.$WEAPPS_COMP) {
        fn = fn.bind(codeContext.$WEAPPS_COMP);
      }
      resolvedProps[prop] = fn(forItems, codeContext?.event, dataContext, $w);
    } catch (e) {
      console.error('Error resolving data binding', prop, dataBinds[prop], e);
      if (throwError) {
        throw e;
      }
    }
  }
  return resolvedProps;
}

/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj = {}, context) {
  return Object.keys(funcObj).reduce((obj, fnName) => {
    obj[fnName] = funcObj[fnName]?.bind?.(context);
    return obj;
  }, {});
}

export function kebabCase(str) {
  return str.replace(/[A-Z]/g, (match) => {
    return `-${match.toLowerCase()}`;
  });
}

export function camelcase(str, firstUpperCase = false) {
  str = str.replace(/[_-]([a-z])/g, (l) => {
    return l[1].toUpperCase();
  });

  if (firstUpperCase) str = str.charAt(0).toUpperCase() + str.slice(1);

  return str;
}

export const isEmptyObj = (obj) => {
  if (!isPlainObject(obj)) {
    return false;
  }
  for (const i in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, i)) {
      return false;
    }
  }
  return true;
};

export function isPlainObject(src) {
  return Object.prototype.toString.call(src) === '[object Object]';
}

const SCOPE_SLOT_COMPONENT_LIB = ['tea_basis', 'tea_shop', 'crm_basis'];

const SCOPE_SLOT_MAP = SCOPE_SLOT_COMPONENT_LIB.reduce((map, lib) => {
  map[`${lib}:TableMatching`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TableExpanded`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:TabsTable`] = {
    headerSlot: true,
    recordSlot: true,
  };

  map[`${lib}:Table`] = {
    headerSlot: true,
    recordSlot: true,
  };

  return map;
}, {});

export function isSlot(comp: IPlatformComponent) {
  return !comp.module && !comp.component;
}

/**
 * 判断是否为作用域插槽
 * 1、根据组件dataForm是否有属性包含 x-slot-scope 属性
 * 2、保持历史穷举法不变
 */
export function isScopeSlot(
  comp: {
    module: string;
    component: string;
  },
  slot: string,
  context?: { componentsInfoMap: any } | any,
) {
  const sourceKey = comp.module && comp.component && `${comp.module}:${comp.component}`;

  let hasXslotScope = false;
  try {
    const dataForm = context?.componentsInfoMap?.[sourceKey]?.dataForm;
    if (typeof dataForm === 'object') {
      hasXslotScope = Object.keys(dataForm).some(
        (prop) => slot.startsWith(`${prop}_`) && dataForm[prop]?.['x-slot-scope'],
      );
    }
  } catch (e) {}

  const map = SCOPE_SLOT_MAP[sourceKey];
  return hasXslotScope || map?.[slot];
}

export function findLoginPage(app = _app) {
  const { pages = [] } = app.__internal__.getConfig();
  return pages.find((item) => item.type === 'login');
}

let _AUTH_CONFIG_CACHE = null;
export async function getAuthConfig(app = _app) {
  if (_AUTH_CONFIG_CACHE) {
    return _AUTH_CONFIG_CACHE;
  }
  try {
    const res = await app.cloud.callWedaApi({
      action: 'DescribeRuntimeResourceStrategy',
      data: {
        ResourceType: 'app',
        ResourceId: app.id,
      },
    });
    const settingData = {};
    // 云api不支持map只能传字符串，需要转换
    res.forEach((item) => {
      settingData[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
    });
    _AUTH_CONFIG_CACHE = settingData;
    return _AUTH_CONFIG_CACHE;
  } catch (e) {
    return {
      NeedLogin: false,
      RejectStrategy: 'show_warning',
    };
  }
}

let _AUTH_CACHE_MAP = {};
async function getAccessPermission(app, appId, pageId) {
  const cacheKey = `${appId}-${pageId}`;
  if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
    return _AUTH_CACHE_MAP[cacheKey];
  }

  const res = await app.cloud.callWedaApi({
    action: 'DescribeResourcesPermission',
    data: {
      ResourceType: `app`,
      ResourceIdList: [cacheKey],
      AppResourceId: appId,
    },
  });
  if (Array.isArray(res)) {
    const resourceInfo = res.find((i) => i.ResourceId === cacheKey);
    const isAccess = !!resourceInfo?.IsAccess;
    _AUTH_CACHE_MAP[cacheKey] = isAccess;
    return isAccess;
  } else {
    throw new Error(`鉴权接口返回参数不正确 - ${JSON.stringify(res)}`);
  }
}

export async function checkAnonymous() {
  // 用户是否非匿名登录前端判断条件：有accessToken且scope不为anonymous
  // 此分支逻辑本不应该前端判断是否登录，历史原因后端短期内搞不定，后续后端优化后删除
  let isAnonymous = true;
  try {
    const [scope, { accessToken }] = await Promise.all([loginScope(), getAccessToken()]);
    if (accessToken && scope !== 'anonymous') {
      isAnonymous = false;
    }
  } catch (e) {}
  return isAnonymous;
}

export function formatErrorMsg(e) {
  let msg = e?.error_description || e?.message;
  const uri = e?.error_uri;
  msg = `${msg}${uri ? `-${uri}` : ''}`;
  if (msg.startsWith('Failed to fetch')) {
    msg = `网络故障, 请检查本地网络连接是否正常(${msg})`;
  }
  return msg;
}

/**
 * 检查页面权限
 **/
export async function checkAuth(app, appId, $page, generateLoginUrl?: GenerateLoginUrlType) {
  app.showNavigationBarLoading();
  const { loginConfigVersion } = app.__internal__.getConfig();
  const requestList = [getAccessPermission(app, appId, $page.id)];
  // 有登录配置版本号时，才拉取自定义登录配置进行校验
  if (loginConfigVersion) {
    requestList.push(getAuthConfig(app));
    requestList.push(checkAnonymous());
  }
  try {
    const [isAccess, authConfig, isAnonymous] = await Promise.all(requestList);
    app.hideNavigationBarLoading();

    if (loginConfigVersion && isAnonymous) {
      if (authConfig.NeedLogin || (!isAccess && authConfig.RejectStrategy === 'to_login')) {
        redirectToLogin(generateLoginUrl);
        return false;
      }
    }

    if (!isAccess) {
      app.showToast({
        title: '页面无访问权限',
        icon: 'error',
      });
    }
    return isAccess;
  } catch (e) {
    app.showModal({
      title: '页面鉴权失败',
      content: formatErrorMsg(e),
      showCancel: false,
    });
    // @ts-ignore
    window._aegis?.error?.('CUSTOM_ERROR:', e);
    return false;
  }
}

export type GenerateLoginUrlType = (ctx: { app: IAppAPI }, originFn: () => string) => string;

export const defaultGenerateLoginUrl: GenerateLoginUrlType = () => {
  return getLoginUrl();
};

function getLoginUrl() {
  const app = _app;
  const { envId, id, clientId, loginConfigVersion } = app.__internal__.getConfig();
  return `${location.origin}/__auth/?redirect_uri=${encodeURIComponent(
    location.href,
  )}&app_id=${id}&env_id=${envId}&client_id=${clientId}&config_version=${loginConfigVersion}`;
}

export async function redirectToLogin(generateLoginUrl: GenerateLoginUrlType) {
  // 去登录则清空权限缓存。
  _AUTH_CACHE_MAP = {};
  const app = _app;

  const { loginConfigVersion } = app.__internal__.getConfig();
  // 有登录配置版本号时，自定义登录开启，则跳转到自定义登录页面
  if (loginConfigVersion) {
    const url = generateLoginUrl({ app }, getLoginUrl);
    app.redirectTo({
      mode: 'web',
      params: {},
      url,
    });
  } else {
    app.showToast({
      title: '用户未登录',
      icon: 'error',
    });
  }
}

export function generateDataContext(widget) {
  const dataContext = {};
  while (widget?._scope) {
    const current = widget;
    // 此处采用代理的方式，是为了可以获取到最新的 _scope.dataContext 防止 dataContext 引用被重新赋值
    Object.defineProperty(dataContext, current._scope.id, {
      get() {
        return current._scope.dataContext;
      },
    });
    widget = widget.parent;
  }
  return dataContext;
}
